Methods for NNGP and Derived Objects
summary.NNGP.RdMethods for extracting information from fitted NNGP model of class
NNGP and predict.NNGP objects from predict.
Arguments
- object, x
object of class
NNGPorpredict.NNGP.- sub.sample
an optional list that specifies the samples to included in the summary or composition sampling. Valid tags are
start,end, andthin. Given the value associated with the tags, the sample subset is selected usingseq(as.integer(start), as.integer(end), by=as.integer(thin)). The default values arestart=floor(0.5*n.samples),end=n.samplesandthin=1.- quantiles
for
summary, posterior distribution quantiles to compute.- digits
for
summary, number of digits to report in summary.- ...
currently not used.